// TextureOffset.cpp : Defines the entry point for the console application.
//
//mail to ngoquocvinh@yahoo.com or ngoquocvinhCUDA@gmail.com

#include "stdafx.h"
#include <stdio.h>
#include <cutil.h>

texture<int, 1, cudaReadModeElementType> texRef1;
texture<int, 1, cudaReadModeElementType> texRef2;
__global__ void TextureOffsetTest(int *outData1, int *outData2, int elements, size_t offset0, size_t offset1) 
{
  int indexX = blockIdx.x * blockDim.x + threadIdx.x;
  int indexOfTexRef1 = indexX + offset0 / sizeof(int);
  int indexOfTexRef2 = indexX + offset1 / sizeof(int);
  if (indexX < elements) {
    outData1[indexX] = tex1Dfetch(texRef1, indexOfTexRef1) + 1;
    outData2[indexX] = tex1Dfetch(texRef2, indexOfTexRef2) + 2;
  }
}

int _tmain(int argc, _TCHAR* argv[])
{
  int elements = 20;
  int *hostIn = (int*)malloc(elements * 2 * sizeof(int));
  for (int i = 0; i < elements; i++) {
    hostIn[i] = 10;
    hostIn[i + elements] = 20;
  }
  int *hostOut1 = (int*)malloc(elements * sizeof(int));
  int *hostOut2 = (int*)malloc(elements * sizeof(int));

  int *devData1 = NULL;
  cudaMalloc((void**)&devData1, sizeof(int) * elements);
  int *devData2 = NULL;
  cudaMalloc((void**)&devData2, sizeof(int) * elements);
	
  int *devIn = NULL;
  cudaMalloc((void**)&devIn, sizeof(int) * elements * 2);
  cudaMemcpy(devIn, hostIn, sizeof(int) * elements * 2, cudaMemcpyHostToDevice);
  size_t offset0;
  cudaBindTexture(&offset0, texRef1, devIn, sizeof(int) * elements);
  size_t offset1;
  cudaBindTexture(&offset1, texRef2, devIn + elements, sizeof(int) * elements);

  dim3 block(16, 1, 1);
  dim3 grid((elements + block.x - 1) / block.x, 1, 1);
  TextureOffsetTest<<<grid, block>>>(devData1, devData2, elements, offset1);
  cudaMemcpy(hostOut1, devData1, sizeof(int) * elements, cudaMemcpyDeviceToHost);
  cudaMemcpy(hostOut2, devData2, sizeof(int) * elements, cudaMemcpyDeviceToHost);
  for (int i = 0; i < elements; i++) {
    printf("%d, %d	%d, %d\n", hostIn[i], hostOut1[i], hostIn[i + elements], hostOut2[i]);
  }
  cudaFree(devData1);
  cudaFree(devData2);
  cudaFree(devIn);
  free(hostIn);
  free(hostOut1);
  free(hostOut2);
  getchar();

  return 0;
}

